/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.configuration;

import java.io.Serializable;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;

import org.ow2.petals.clientserverapi.jbi.messaging.RouterService;
import org.ow2.petals.clientserverapi.system.persistence.PersistenceService;
import org.ow2.petals.clientserverapi.tools.ws.WebServiceManager;

/**
 * This class holds the overall configuration of a PEtALS container. 
 * <p>
 * It stores both the local topology information and the global server properties
 * information.<br>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 * </p>
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @since Petals 2.0
 */
public final class ContainerConfiguration implements Serializable {

    /**
     * Container state enumeration.
     */
    public static enum ContainerState {

        /**
         * Started state
         */
        STARTED,
        /**
         * Stopped state
         */
        STOPPED,
        /**
         * unknown state
         */
        UNKNOWN
    }

    /**
     * Registry (and maybe more) policy - related stuff.
     */
    public static enum RegistryMode {

        /**
         * Standalone mode.
         */
        STANDALONE,
        /**
         * Peer mode, used for flooding based strategy.
         */
        PEER,
        /**
         * Master mode for master / slave based strategy.
         */
        MASTER,
        /**
         * Slave mode for master / slave based strategy.
         */
        SLAVE;
    };

    /**
     * the "unique" ID for serialization
     */
    private static final long serialVersionUID = 6543213857L;;

    // ********** Global ************//
    /**
     * The location of the local configuration
     */
    private URL serverPropertiesUrl;

    /**
     * Subdomain name.
     */
    private String subdomainName;

    /**
     * Container name.
     */
    private String name;

    /**
     * Container description
     */
    private String description;

    /**
     * Host of the container.
     */
    private String host;

    /**
     * State of the container.
     */
    private ContainerState state = ContainerState.UNKNOWN;

    /**
     * Data root path
     */
    private String dataRootPath;

    /**
     * Repository directory path.
     */
    private String repositoryDirectoryPath;

    /**
     * The work directory where temporary things are stored
     */
    private String workDirectoryPath;

    /**
     * Exchange validation.<br>
     * If <code>true</code>, used by the Address resolver to know if a component
     * is agree to receive a exchange from a consumer.
     */
    private boolean exchangeValidation;

    /**
     * The timeout for processing the installation and deployment tasks.
     */
    private long taskTimeout;

    /**
     * Flag indicating if the JBI classLoaders must be isolated from the
     * container ClassLoader.
     */
    private boolean isolateJBIClassLoaders;

    /**
     * Flag indicating if the autoloader is activated or not.
     */
    private boolean activateAutoloader = true;

    // ************ Admin **************//

    /**
     * Login to connect to the node.
     */
    private String user;

    /**
     * Password to connect to the node.
     */
    private String password;

    /**
     * Port of the RMI adaptor for JMX.
     */
    private int jmxRMIConnectorPort;

    /**
     * Web Service Service
     */
    private int webservicePort = WebServiceManager.DEFAULT_PORT;

    private String webservicePrefix = WebServiceManager.DEFAULT_PREFIX;

    // ***** Router *****//

    /**
     * Router Quality of Service. {@see RouterService}
     */
    private String routerQOS = RouterService.DEFAULT_POLICY;

    /**
     * Router Strategy. {@see EndpointResolverModule}
     */
    private String routerStrategy = RouterService.DEFAULT_PLATFORM_STRATEGY;

    /**
     * Number of attempt to send a message
     */
    private short routerSendAttempt = 2;

    /**
     * Delay between each send attempt
     */
    private int routerSendDelay = 1000;

    // ***** Topology *****//

    /**
     * The topology pass-phrase
     */
    private String topologyPassPhrase = null;

    // ***** Transporter *****//

    /**
     * Port of the TCP communication.
     */
    private int tcpPort;

    /**
     * The SSL key password
     */
    private String sslKeyPassword;

    /**
     * Max size of the transporter queues. Default value: 10000
     */
    private int transporterQueueMaxSize = 10000;

    /**
     * Offering timeout of component transporter queues, in milliseconds.
     * Default value : 2500ms
     */
    private long transporterQueueOfferingTimeout = 2500;

    /**
     * Number of message that can be received via TCP at the same time
     */
    private int tcpReceivers = 10;

    /**
     * Number of message that can be send via TCP at the same time, per
     * component
     */
    private int tcpSenders = 10;

    /**
     * TCP Connection timeout
     */
    private long tcpConnectionTimeout = 5000;

    /**
     * TCP send timeout
     */
    private long tcpSendTimeout = 5000;

    /**
     * The delay before running the 'sender' eviction thread
     */
    private long tcpSenderEvictorDelay = 60000;

    /**
     * The delay before an idle 'sender' is set evictable
     */
    private long tcpSenderEvictableDelay = 60000;

    // ***** Security *****//

    /**
     * The SSL Keystore file
     */
    private String sslKeystore;

    /**
     * The SSL Keystore password
     */
    private String sslKeystorePassword;

    /**
     * The SSL Truststore file
     */
    private String sslTruststore;

    /**
     * The SSL Truststore password
     */
    private String sslTruststorePassword;

    // ***** Registry *****//

    /**
     * The Registry Service
     */
    private int registryPort;

    /**
     * The registry mode.
     */
    private RegistryMode registryMode;

    /**
     * The registry synchro period.
     */
    private int registrySynchroPeriod;

    /**
     * The registry transport timeout.
     */
    private int registryTransportTimeout;

    /**
     * The registry repository.
     */
    private String registryRepository;

    /**
     * The registry database driver.
     */
    private String registryDatabaseDriver = PersistenceService.HSQLDB_DRIVER;

    /**
     * The registry database location.
     */
    private String registryDatabaseUrl;

    /**
     * The registry database user.
     */
    private String registryDatabaseUser = PersistenceService.DEFAULT_USER;

    /**
     * The registry database password.
     */
    private String registryDatabasePassword = PersistenceService.DEFAULT_PASSWORD;

    /**
     * The registry database dialect.
     */
    private String registryDatabaseDialect;

    // ***** Persistence *****//

    /**
     * The persistence duration.
     */
    private long persistenceDuration = PersistenceService.DEFAULT_DURATION;
    
    /**
     * The persistence duration.
     */
    private int recoveryCorePoolSize = 5;
    
    /**
     * The persistence duration.
     */
    private int recoveryMaxPoolSize = 5;
    
    /**
     * The persistence duration.
     */
    private long recoveryKeepAliveTime = 60;

    /**
     * The persistence SQL request fetch size.
     */
    private int persistenceFetchSize = PersistenceService.DEFAULT_FETCHSIZE;

    /**
     * The additional configuration in the server local configuration (ie. the
     * content of '<code>server.properties</code>' or equivalent) defined by the
     * user. Get all the '<code>user.*</code>' key/value pairs.
     */
    private Map<String, String> userConfiguration;

    /**
     * The additional configuration in the server local configuration (ie. the
     * content of '<code>server.properties</code>' or equivalent) that is
     * present but a priori not used.
     */
    private Map<String, String> extraConfiguration;

    /**
     * @return the persistenceFetchSize
     */
    public int getPersistenceFetchSize() {
        return persistenceFetchSize;
    }

    /**
     * @param persistenceFetchSize
     *            the persistenceFetchSize to set
     */
    public void setPersistenceFetchSize(int persistenceFetchSize) {
        this.persistenceFetchSize = persistenceFetchSize;
    }

    /**
     * Default constructor
     */
    public ContainerConfiguration() {
        super();
    }

    /**
     * @return the serialVersionUID
     */
    public static long getSerialVersionUID() {
        return serialVersionUID;
    }

    /**
     * @return the description
     */
    public String getDescription() {
        return this.description;
    }

    /**
     * @return the host
     */
    public String getHost() {
        return this.host;
    }

    /**
     * @return the jmxRMIConnectorPort
     */
    public int getJmxRMIConnectorPort() {
        return this.jmxRMIConnectorPort;
    }

    /**
     * @return the name
     */
    public String getName() {
        return this.name;
    }

    /**
     * @return the password
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * 
     * @return
     */
    public long getPersistenceDuration() {
        return persistenceDuration;
    }

    /**
     * @return the registryDatabaseDialect
     */
    public String getRegistryDatabaseDialect() {
        return this.registryDatabaseDialect;
    }

    /**
     * @return the registryDatabaseDriver
     */
    public String getRegistryDatabaseDriver() {
        return this.registryDatabaseDriver;
    }

    /**
     * @return the registryDatabasePassword
     */
    public String getRegistryDatabasePassword() {
        return this.registryDatabasePassword;
    }

    /**
     * @return the registryDatabaseUrl
     */
    public String getRegistryDatabaseUrl() {
        return this.registryDatabaseUrl;
    }

    /**
     * @return the registryDatabaseUser
     */
    public String getRegistryDatabaseUser() {
        return this.registryDatabaseUser;
    }

    /**
     * @return the registryMode
     */
    public RegistryMode getRegistryMode() {
        return this.registryMode;
    }

    /**
     * @return the registryPort
     */
    public int getRegistryPort() {
        return this.registryPort;
    }

    /**
     * @return the registryRepository
     */
    public String getRegistryRepository() {
        return this.registryRepository;
    }

    /**
     * @return the registrySynchroPeriod
     */
    public int getRegistrySynchroPeriod() {
        return this.registrySynchroPeriod;
    }

    /**
     * @return the registryTransportTimeout
     */
    public int getRegistryTransportTimeout() {
        return this.registryTransportTimeout;
    }

    /**
     * @return the repositoryDirectoryPath
     */
    public String getRepositoryDirectoryPath() {
        return this.repositoryDirectoryPath;
    }

    /**
     * @return the routerQOS
     */
    public String getRouterQOS() {
        return this.routerQOS;
    }

    /**
     * @return the routerSendAttempt
     */
    public short getRouterSendAttempt() {
        return this.routerSendAttempt;
    }

    /**
     * @return the routerSendDelay
     */
    public int getRouterSendDelay() {
        return this.routerSendDelay;
    }

    /**
     * @return the routerStrategy
     */
    public String getRouterStrategy() {
        return this.routerStrategy;
    }

    /**
     * @return the sslKeyPassword
     */
    public String getSSLKeyPassword() {
        return this.sslKeyPassword;
    }

    /**
     * @return the sslKeystore
     */
    public String getSSLKeystore() {
        return this.sslKeystore;
    }

    /**
     * @return the sslKeystorePassword
     */
    public String getSSLKeystorePassword() {
        return this.sslKeystorePassword;
    }

    /**
     * @return the sslTruststore
     */
    public String getSSLTruststore() {
        return this.sslTruststore;
    }

    /**
     * @return the sslTruststorePassword
     */
    public String getSSLTruststorePassword() {
        return this.sslTruststorePassword;
    }

    /**
     * @return the state
     */
    public ContainerState getState() {
        return this.state;
    }

    /**
     * @return the subdomainName
     */
    public String getSubdomainName() {
        return this.subdomainName;
    }

    /**
     * @return the taskTimeout
     */
    public long getTaskTimeout() {
        return this.taskTimeout;
    }

    /**
     * @return the tcpConnectionTimeout
     */
    public long getTCPConnectionTimeout() {
        return this.tcpConnectionTimeout;
    }

    /**
     * @return the tcpPort
     */
    public int getTCPPort() {
        return this.tcpPort;
    }

    /**
     * @return the tcpReceivers
     */
    public int getTCPReceivers() {
        return this.tcpReceivers;
    }

    /**
     * @return the tcpSenderEvictableDelay
     */
    public long getTCPSenderEvictableDelay() {
        return this.tcpSenderEvictableDelay;
    }

    /**
     * @return the tcpSenderEvictorDelay
     */
    public long getTCPSenderEvictorDelay() {
        return this.tcpSenderEvictorDelay;
    }

    /**
     * @return the tcpSenders
     */
    public int getTCPSenders() {
        return this.tcpSenders;
    }

    /**
     * @return the tcpSendTimeout
     */
    public long getTCPSendTimeout() {
        return this.tcpSendTimeout;
    }

    public String getTopologyPassPhrase() {
        return this.topologyPassPhrase;
    }

    /**
     * @return the user
     */
    public String getUser() {
        return this.user;
    }

    /**
     * @return the userConfiguration
     */
    public Map<String, String> getUserConfiguration() {
        return this.userConfiguration;
    }

    /**
     * @return The extra properties. A properties is an extra property if not
     *         directly used by the micro-kernel but by an extension or by an
     *         dedicated Fractal component implementation. The {@link Map}
     *         returned is never <code>null</code>, can be empty.
     */
    public Map<String, String> getExtraConfiguration() {
        return this.extraConfiguration;
    }

    /**
     * @return the webservicePort
     */
    public int getWebservicePort() {
        return this.webservicePort;
    }

    /**
     * @return the webservicePrefix
     */
    public String getWebservicePrefix() {
        return this.webservicePrefix;
    }

    /**
     * @return the workDirectoryPath
     */
    public String getWorkDirectoryPath() {
        return this.workDirectoryPath;
    }

    /**
     * @return the activateAutoloader
     */
    public boolean isActivateAutoloader() {
        return this.activateAutoloader;
    }

    /**
     * @return the exchangeValidation
     */
    public boolean isExchangeValidation() {
        return this.exchangeValidation;
    }

    public boolean isInFloodingMode() {
        return this.isPeer();
    }

    public boolean isInMasterSlaveMode() {
        return this.isMaster() || this.isSlave();
    }

    public boolean isInStandaloneMode() {
        return this.isStandalone();
    }

    /**
     * @return the isolateJBIClassLoaders
     */
    public boolean isIsolateJBIClassLoaders() {
        return this.isolateJBIClassLoaders;
    }

    public boolean isMaster() {
        return this.getRegistryMode().equals(RegistryMode.MASTER);
    }

    public boolean isPeer() {
        return this.getRegistryMode().equals(RegistryMode.PEER);
    }

    public boolean isSlave() {
        return this.getRegistryMode().equals(RegistryMode.SLAVE);
    }

    public boolean isStandalone() {
        return this.getRegistryMode().equals(RegistryMode.STANDALONE);
    }

    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Container ").append(this.getName()).append(" on ")
                .append(this.getHost()).append(" in sub-domain ").append(this.getSubdomainName());
        if (this.getDescription() != null) {
            toStringBuilder.append("\n\t\t\tDescription :\n\t\t\t  ").append(this.getDescription());
        }
        toStringBuilder.append("\n\t\t\t  - RepositoryDirectory:").append(
                this.getRepositoryDirectoryPath());
        toStringBuilder.append("\n\t\t\t  - WorkDirectory:").append(
                this.getWorkDirectoryPath());
        toStringBuilder.append("\n\t\t\t  - ExchangeValidation:").append(
                this.isExchangeValidation());
        toStringBuilder.append("\n\t\t\t  - RouterQOS:").append(this.getRouterQOS());
        toStringBuilder.append("\n\t\t\t  - RoutingStrategy:").append(this.getRouterStrategy());
        toStringBuilder.append("\n\t\t\t  - TaskTimeout:").append(this.getTaskTimeout());
        toStringBuilder.append("\n\t\t\t  - IsolateClassLoaders:").append(
                this.isIsolateJBIClassLoaders());
        toStringBuilder.append("\n\t\t\t  - ActivateAutoloader:").append(
                this.isActivateAutoloader());
        toStringBuilder.append("\n\t\t\t  - User:").append(
                (this.getUser() != null) ? this.getUser() : "none");
        if (this.getPassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getPassword().length());
            while (hiddenPassword.length() < this.getPassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - Password:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - Password:none");
        }
        toStringBuilder.append("\n\t\t\t  - JMXRMIConnectionPort:").append(
                this.getJmxRMIConnectorPort());
        toStringBuilder.append("\n\t\t\t  - TCPPort:").append(
                (this.getTCPPort() != 0) ? this.getTCPPort() : "none");
        if (this.getSSLKeyPassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLKeyPassword().length());
            while (hiddenPassword.length() < this.getSSLKeyPassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - sslKeyPassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - sslKeyPassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - Keystore:").append(
                (this.getSSLKeystore() != null) ? this.getSSLKeystore() : "none");
        if (this.getSSLKeystorePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLKeystorePassword().length());
            while (hiddenPassword.length() < this.getSSLKeystorePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - KeystorePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - KeystorePassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - Truststore:").append(
                (this.getSSLTruststore() != null) ? this.getSSLTruststore() : "none");
        if (this.getSSLTruststorePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLTruststorePassword().length());
            while (hiddenPassword.length() < this.getSSLTruststorePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - TruststorePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - TruststorePassword:none");
        }
        if (this.getTCPPort() != 0) {
            toStringBuilder.append("\n\t\t\t  - TCPReceivers:").append(this.getTCPReceivers());
            toStringBuilder.append("\n\t\t\t  - TCPSenders:").append(this.getTCPSenders());
            toStringBuilder.append("\n\t\t\t  - TCPConnectionTimeout:").append(
                    this.getTCPConnectionTimeout());
            toStringBuilder.append("\n\t\t\t  - TCPSendTimeout:").append(this.getTCPSendTimeout());
            toStringBuilder.append("\n\t\t\t  - TCPSenderEvictorDelay:").append(
                    this.getTCPSenderEvictorDelay());
            toStringBuilder.append("\n\t\t\t  - TCPSenderEvictableDelay:").append(
                    this.getTCPSenderEvictableDelay());
        }
        toStringBuilder.append("\n\t\t\t  - State:").append(this.getState().name());
        toStringBuilder.append("\n\t\t\t  - RegistryPort:").append(this.getRegistryPort());
        toStringBuilder.append("\n\t\t\t  - RegistryMode:").append(this.getRegistryMode());
        toStringBuilder.append("\n\t\t\t  - RegistrySynchroPeriod:").append(
                this.getRegistrySynchroPeriod());
        toStringBuilder.append("\n\t\t\t  - RegistryTransportTimeout:").append(
                this.getRegistryTransportTimeout());
        toStringBuilder.append("\n\t\t\t  - RegistryRepository:").append(
                this.getRegistryRepository());
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseDriver:").append(
                this.getRegistryDatabaseDriver());
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseUrl:").append(
                this.getRegistryDatabaseUrl());
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseUser:").append(
                this.getRegistryDatabaseUser());
        if (this.getRegistryDatabasePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getRegistryDatabasePassword()
                    .length());
            while (hiddenPassword.length() < this.getRegistryDatabasePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - RegistryDatabasePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - RegistryDatabasePassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseDialect:").append(
                this.getRegistryDatabaseDialect());
        toStringBuilder.append("\n\t\t\t  - WebServicePort:").append(this.getWebservicePort());
        toStringBuilder.append("\n\t\t\t  - WebServicePrefix:").append(this.getWebservicePrefix());
        toStringBuilder.append("\n\t\t\t  - Persistence duration:").append(
                this.getPersistenceDuration());

        if (this.getUserConfiguration() != null) {
            StringBuffer userBuffer = new StringBuffer();
            Iterator<String> iter = this.getUserConfiguration().keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                userBuffer.append("\n\t\t\t\t").append(key).append('=')
                        .append(this.getUserConfiguration().get(key));
            }
            toStringBuilder.append("\n\t\t\t  - User properties:").append(userBuffer.toString());
        }

        return toStringBuilder.toString();
    }

    /**
     * @param activateAutoloader
     *            the activateAutoloader to set
     */
    public void setActivateAutoloader(boolean activateAutoloader) {
        this.activateAutoloader = activateAutoloader;
    }

    /**
     * @param description
     *            the description to set
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @param exchangeValidation
     *            the exchangeValidation to set
     */
    public void setExchangeValidation(boolean exchangeValidation) {
        this.exchangeValidation = exchangeValidation;
    }

    /**
     * @param host
     *            the host to set
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * @param isolateJBIClassLoaders
     *            the isolateJBIClassLoaders to set
     */
    public void setIsolateJBIClassLoaders(boolean isolateJBIClassLoaders) {
        this.isolateJBIClassLoaders = isolateJBIClassLoaders;
    }

    /**
     * @param jmxRMIConnectorPort
     *            the jmxRMIConnectorPort to set
     */
    public void setJmxRMIConnectorPort(int jmxRMIConnectorPort) {
        this.jmxRMIConnectorPort = jmxRMIConnectorPort;
    }

    /**
     * @param name
     *            the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @param password
     *            the password to set
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * 
     * @param persistenceDuration
     */
    public void setPersistenceDuration(long persistenceDuration) {
        this.persistenceDuration = persistenceDuration;
    }

    /**
     * @param registryDatabaseDialect
     *            the registryDatabaseDialect to set
     */
    public void setRegistryDatabaseDialect(String registryDatabaseDialect) {
        this.registryDatabaseDialect = registryDatabaseDialect;
    }

    /**
     * @param registryDatabaseDriver
     *            the registryDatabaseDriver to set
     */
    public void setRegistryDatabaseDriver(String registryDatabaseDriver) {
        this.registryDatabaseDriver = registryDatabaseDriver;
    }

    /**
     * @param registryDatabasePassword
     *            the registryDatabasePassword to set
     */
    public void setRegistryDatabasePassword(String registryDatabasePassword) {
        this.registryDatabasePassword = registryDatabasePassword;
    }

    /**
     * @param registryDatabaseUrl
     *            the registryDatabaseUrl to set
     */
    public void setRegistryDatabaseUrl(String registryDatabaseUrl) {
        this.registryDatabaseUrl = registryDatabaseUrl;
    }

    /**
     * @param registryDatabaseUser
     *            the registryDatabaseUser to set
     */
    public void setRegistryDatabaseUser(String registryDatabaseUser) {
        this.registryDatabaseUser = registryDatabaseUser;
    }

    /**
     * @param registryMode
     *            the registryMode to set
     */
    public void setRegistryMode(RegistryMode registryMode) {
        this.registryMode = registryMode;
    }

    /**
     * @param registryPort
     *            the registryPort to set
     */
    public void setRegistryPort(int registryPort) {
        this.registryPort = registryPort;
    }

    /**
     * @param registryRepository
     *            the registryRepository to set
     */
    public void setRegistryRepository(String registryRepository) {
        this.registryRepository = registryRepository;
    }

    /**
     * @param registrySynchroPeriod
     *            the registrySynchroPeriod to set
     */
    public void setRegistrySynchroPeriod(int registrySynchroPeriod) {
        this.registrySynchroPeriod = registrySynchroPeriod;
    }

    /**
     * @param registryTransportTimeout
     *            the registryTransportTimeout to set
     */
    public void setRegistryTransportTimeout(int registryTransportTimeout) {
        this.registryTransportTimeout = registryTransportTimeout;
    }

    /**
     * @param repositoryDirectoryPath
     *            the repositoryDirectoryPath to set
     */
    public void setRepositoryDirectoryPath(String repositoryDirectoryPath) {
        this.repositoryDirectoryPath = repositoryDirectoryPath;
    }

    /**
     * @param routerQOS
     *            the routerQOS to set
     */
    public void setRouterQOS(String routerQOS) {
        this.routerQOS = routerQOS;
    }

    /**
     * @param routerSendAttempt
     *            the routerSendAttempt to set
     */
    public void setRouterSendAttempt(short routerSendAttempt) {
        this.routerSendAttempt = routerSendAttempt;
    }

    /**
     * @param routerSendDelay
     *            the routerSendDelay to set
     */
    public void setRouterSendDelay(int routerSendDelay) {
        this.routerSendDelay = routerSendDelay;
    }

    /**
     * @param routerStrategy
     *            the routerStrategy to set
     */
    public void setRouterStrategy(String routerStrategy) {
        this.routerStrategy = routerStrategy;
    }

    /**
     * @param sslKeyPassword
     *            the sslKeyPassword to set
     */
    public void setSslKeyPassword(String sslKeyPassword) {
        this.sslKeyPassword = sslKeyPassword;
    }

    /**
     * @param sslKeyPassword
     *            the sslKeyPassword to set
     */
    public void setSSLKeyPassword(String sslKeyPassword) {
        this.setSslKeyPassword(sslKeyPassword);
    }

    /**
     * @param sslKeystore
     *            the sslKeystore to set
     */
    public void setSslKeystore(String sslKeystore) {
        this.sslKeystore = sslKeystore;
    }

    /**
     * @param sslKeystore
     *            the sslKeystore to set
     */
    public void setSSLKeystore(String sslKeystore) {
        this.setSslKeystore(sslKeystore);
    }

    /**
     * @param sslKeystorePassword
     *            the sslKeystorePassword to set
     */
    public void setSslKeystorePassword(String sslKeystorePassword) {
        this.sslKeystorePassword = sslKeystorePassword;
    }

    /**
     * @param sslKeystorePassword
     *            the sslKeystorePassword to set
     */
    public void setSSLKeystorePassword(String sslKeystorePassword) {
        this.setSslKeystorePassword(sslKeystorePassword);
    }

    /**
     * @param sslTruststore
     *            the sslTruststore to set
     */
    public void setSslTruststore(String sslTruststore) {
        this.sslTruststore = sslTruststore;
    }

    /**
     * @param sslTruststore
     *            the sslTruststore to set
     */
    public void setSSLTruststore(String sslTruststore) {
        this.setSslTruststore(sslTruststore);
    }

    /**
     * @param sslTruststorePassword
     *            the sslTruststorePassword to set
     */
    public void setSslTruststorePassword(String sslTruststorePassword) {
        this.sslTruststorePassword = sslTruststorePassword;
    }

    /**
     * @param sslTruststorePassword
     *            the sslTruststorePassword to set
     */
    public void setSSLTruststorePassword(String sslTruststorePassword) {
        this.setSslTruststorePassword(sslTruststorePassword);
    }

    /**
     * @param state
     *            the state to set
     */
    public void setState(ContainerState state) {
        this.state = state;
    }

    /**
     * @param subdomainName
     *            the subdomainName to set
     */
    public void setSubdomainName(String subdomainName) {
        this.subdomainName = subdomainName;
    }

    /**
     * @param taskTimeout
     *            the taskTimeout to set
     */
    public void setTaskTimeout(long taskTimeout) {
        this.taskTimeout = taskTimeout;
    }

    /**
     * @param tcpConnectionTimeout
     *            the tcpConnectionTimeout to set
     */
    public void setTcpConnectionTimeout(long tcpConnectionTimeout) {
        this.tcpConnectionTimeout = tcpConnectionTimeout;
    }

    /**
     * @param tcpConnectionTimeout
     *            the tcpConnectionTimeout to set
     */
    public void setTCPConnectionTimeout(long tcpConnectionTimeout) {
        this.setTcpConnectionTimeout(tcpConnectionTimeout);
    }

    /**
     * @param tcpPort
     *            the tcpPort to set
     */
    public void setTcpPort(int tcpPort) {
        this.tcpPort = tcpPort;
    }

    /**
     * @param tcpPort
     *            the tcpPort to set
     */
    public void setTCPPort(int tcpPort) {
        this.setTcpPort(tcpPort);
    }

    /**
     * @param tcpReceivers
     *            the tcpReceivers to set
     */
    public void setTcpReceivers(int tcpReceivers) {
        this.tcpReceivers = tcpReceivers;
    }

    /**
     * @param tcpReceivers
     *            the tcpReceivers to set
     */
    public void setTCPReceivers(int tcpReceivers) {
        this.setTcpReceivers(tcpReceivers);
    }

    /**
     * @param tcpSenderEvictableDelay
     *            the tcpSenderEvictableDelay to set
     */
    public void setTcpSenderEvictableDelay(long tcpSenderEvictableDelay) {
        this.tcpSenderEvictableDelay = tcpSenderEvictableDelay;
    }

    /**
     * @param tcpSenderEvictableDelay
     *            the tcpSenderEvictableDelay to set
     */
    public void setTCPSenderEvictableDelay(long tcpSenderEvictableDelay) {
        this.setTcpSenderEvictableDelay(tcpSenderEvictableDelay);
    }

    /**
     * @param tcpSenderEvictorDelay
     *            the tcpSenderEvictorDelay to set
     */
    public void setTcpSenderEvictorDelay(long tcpSenderEvictorDelay) {
        this.tcpSenderEvictorDelay = tcpSenderEvictorDelay;
    }

    /**
     * @param tcpSenderEvictorDelay
     *            the tcpSenderEvictorDelay to set
     */
    public void setTCPSenderEvictorDelay(long tcpSenderEvictorDelay) {
        this.setTcpSenderEvictorDelay(tcpSenderEvictorDelay);
    }

    /**
     * @param tcpSenders
     *            the tcpSenders to set
     */
    public void setTcpSenders(int tcpSenders) {
        this.tcpSenders = tcpSenders;
    }

    /**
     * @param tcpSenders
     *            the tcpSenders to set
     */
    public void setTCPSenders(int tcpSenders) {
        this.setTcpSenders(tcpSenders);
    }

    /**
     * @param tcpSendTimeout
     *            the tcpSendTimeout to set
     */
    public void setTcpSendTimeout(long tcpSendTimeout) {
        this.tcpSendTimeout = tcpSendTimeout;
    }

    /**
     * @param tcpSendTimeout
     *            the tcpSendTimeout to set
     */
    public void setTCPSendTimeout(long tcpSendTimeout) {
        this.setTcpSendTimeout(tcpSendTimeout);
    }

    /**
     * 
     * @param topologyPassPhrase
     */
    public void setTopologyPassPhrase(final String topologyPassPhrase) {
        this.topologyPassPhrase = topologyPassPhrase;
    }

    /**
     * @param user
     *            the user to set
     */
    public void setUser(String user) {
        this.user = user;
    }

    /**
     * @param userConfiguration
     *            the userConfiguration to set
     */
    public void setUserConfiguration(Map<String, String> userConfiguration) {
        this.userConfiguration = userConfiguration;
    }

    /**
     * @param extraConfiguration
     *            the extraConfiguration to set
     */
    public void setExtraConfiguration(Map<String, String> extraConfiguration) {
        this.extraConfiguration = extraConfiguration;
    }

    /**
     * @param webservicePort
     *            the webservicePort to set
     */
    public void setWebservicePort(int webservicePort) {
        this.webservicePort = webservicePort;
    }

    /**
     * @param webservicePrefix
     *            the webservicePrefix to set
     */
    public void setWebservicePrefix(String webservicePrefix) {
        this.webservicePrefix = webservicePrefix;
    }

    /**
     * @param workDirectoryPath
     *            the workDirectoryPath to set
     */
    public void setWorkDirectoryPath(String workDirectoryPath) {
        this.workDirectoryPath = workDirectoryPath;
    }

    public int getRecoveryCorePoolSize() {
        return recoveryCorePoolSize;
    }

    public void setRecoveryCorePoolSize(int recoveryCorePoolSize) {
        this.recoveryCorePoolSize = recoveryCorePoolSize;
    }

    public int getRecoveryMaxPoolSize() {
        return recoveryMaxPoolSize;
    }

    public void setRecoveryMaxPoolSize(int recoveryMaxPoolSize) {
        this.recoveryMaxPoolSize = recoveryMaxPoolSize;
    }

    public long getRecoveryKeepAliveTime() {
        return recoveryKeepAliveTime;
    }

    public void setRecoveryKeepAliveTime(long recoveryKeepAliveTime) {
        this.recoveryKeepAliveTime = recoveryKeepAliveTime;
    }

    /**
     * @return the transporterQueueMaxSize
     */
    public int getTransporterQueueMaxSize() {
        return transporterQueueMaxSize;
    }

    /**
     * @param transporterQueueMaxSize
     *            the transporterQueueMaxSize to set
     */
    public void setTransporterQueueMaxSize(int transporterQueueMaxSize) {
        this.transporterQueueMaxSize = transporterQueueMaxSize;
    }

    /**
     * @return the transporterQueueOfferingTimeout
     */
    public long getTransporterQueueOfferingTimeout() {
        return transporterQueueOfferingTimeout;
    }

    /**
     * @param transporterQueueOfferingTimeout
     *            the transporterQueueOfferingTimeout to set
     */
    public void setTransporterQueueOfferingTimeout(long transporterQueueOfferingTimeout) {
        this.transporterQueueOfferingTimeout = transporterQueueOfferingTimeout;
    }

    /**
     * @return the serverPropertiesUrl
     */
    public URL getServerPropertiesUrl() {
        return this.serverPropertiesUrl;
    }

    /**
     * @param serverPropertiesUrl
     *            the serverPropertiesUrl to set
     */
    public void setServerPropertiesUrl(final URL serverPropertiesUrl) {
        this.serverPropertiesUrl = serverPropertiesUrl;
    }

    /**
     * @return the dataRootPath
     */
    public String getDataRootPath() {
        return this.dataRootPath;
    }

    /**
     * @param dataRootPath
     *            the dataRootPath to set
     */
    public void setDataRootPath(final String dataRootPath) {
        this.dataRootPath = dataRootPath;
    }

    /* Generated by Eclipse */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (activateAutoloader ? 1231 : 1237);
        result = prime * result + ((dataRootPath == null) ? 0 : dataRootPath.hashCode());
        result = prime * result + ((description == null) ? 0 : description.hashCode());
        result = prime * result + (exchangeValidation ? 1231 : 1237);
        result = prime * result
                + ((extraConfiguration == null) ? 0 : extraConfiguration.hashCode());
        result = prime * result + ((host == null) ? 0 : host.hashCode());
        result = prime * result + (isolateJBIClassLoaders ? 1231 : 1237);
        result = prime * result + jmxRMIConnectorPort;
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        result = prime * result + ((password == null) ? 0 : password.hashCode());
        result = prime * result + (int) (persistenceDuration ^ (persistenceDuration >>> 32));
        result = prime * result + persistenceFetchSize;
        result = prime * result + recoveryCorePoolSize;
        result = prime * result + (int) (recoveryKeepAliveTime ^ (recoveryKeepAliveTime >>> 32));
        result = prime * result + recoveryMaxPoolSize;
        result = prime * result
                + ((registryDatabaseDialect == null) ? 0 : registryDatabaseDialect.hashCode());
        result = prime * result
                + ((registryDatabaseDriver == null) ? 0 : registryDatabaseDriver.hashCode());
        result = prime * result
                + ((registryDatabasePassword == null) ? 0 : registryDatabasePassword.hashCode());
        result = prime * result
                + ((registryDatabaseUrl == null) ? 0 : registryDatabaseUrl.hashCode());
        result = prime * result
                + ((registryDatabaseUser == null) ? 0 : registryDatabaseUser.hashCode());
        result = prime * result + ((registryMode == null) ? 0 : registryMode.hashCode());
        result = prime * result + registryPort;
        result = prime * result
                + ((registryRepository == null) ? 0 : registryRepository.hashCode());
        result = prime * result + registrySynchroPeriod;
        result = prime * result + registryTransportTimeout;
        result = prime * result
                + ((repositoryDirectoryPath == null) ? 0 : repositoryDirectoryPath.hashCode());
        result = prime * result + ((routerQOS == null) ? 0 : routerQOS.hashCode());
        result = prime * result + routerSendAttempt;
        result = prime * result + routerSendDelay;
        result = prime * result + ((routerStrategy == null) ? 0 : routerStrategy.hashCode());
        result = prime * result
                + ((serverPropertiesUrl == null) ? 0 : serverPropertiesUrl.hashCode());
        result = prime * result + ((sslKeyPassword == null) ? 0 : sslKeyPassword.hashCode());
        result = prime * result + ((sslKeystore == null) ? 0 : sslKeystore.hashCode());
        result = prime * result
                + ((sslKeystorePassword == null) ? 0 : sslKeystorePassword.hashCode());
        result = prime * result + ((sslTruststore == null) ? 0 : sslTruststore.hashCode());
        result = prime * result
                + ((sslTruststorePassword == null) ? 0 : sslTruststorePassword.hashCode());
        result = prime * result + ((state == null) ? 0 : state.hashCode());
        result = prime * result + ((subdomainName == null) ? 0 : subdomainName.hashCode());
        result = prime * result + (int) (taskTimeout ^ (taskTimeout >>> 32));
        result = prime * result + (int) (tcpConnectionTimeout ^ (tcpConnectionTimeout >>> 32));
        result = prime * result + tcpPort;
        result = prime * result + tcpReceivers;
        result = prime * result + (int) (tcpSendTimeout ^ (tcpSendTimeout >>> 32));
        result = prime * result
                + (int) (tcpSenderEvictableDelay ^ (tcpSenderEvictableDelay >>> 32));
        result = prime * result + (int) (tcpSenderEvictorDelay ^ (tcpSenderEvictorDelay >>> 32));
        result = prime * result + tcpSenders;
        result = prime * result
                + ((topologyPassPhrase == null) ? 0 : topologyPassPhrase.hashCode());
        result = prime * result + transporterQueueMaxSize;
        result = prime
                * result
                + (int) (transporterQueueOfferingTimeout ^ (transporterQueueOfferingTimeout >>> 32));
        result = prime * result + ((user == null) ? 0 : user.hashCode());
        result = prime * result + ((userConfiguration == null) ? 0 : userConfiguration.hashCode());
        result = prime * result + webservicePort;
        result = prime * result + ((webservicePrefix == null) ? 0 : webservicePrefix.hashCode());
        result = prime * result + ((workDirectoryPath == null) ? 0 : workDirectoryPath.hashCode());
        return result;
    }

    /* Generated by Eclipse */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof ContainerConfiguration)) {
            return false;
        }
        ContainerConfiguration other = (ContainerConfiguration) obj;
        if (activateAutoloader != other.activateAutoloader) {
            return false;
        }
        if (dataRootPath == null) {
            if (other.dataRootPath != null) {
                return false;
            }
        } else if (!dataRootPath.equals(other.dataRootPath)) {
            return false;
        }
        if (description == null) {
            if (other.description != null) {
                return false;
            }
        } else if (!description.equals(other.description)) {
            return false;
        }
        if (exchangeValidation != other.exchangeValidation) {
            return false;
        }
        if (extraConfiguration == null) {
            if (other.extraConfiguration != null) {
                return false;
            }
        } else if (!extraConfiguration.equals(other.extraConfiguration)) {
            return false;
        }
        if (host == null) {
            if (other.host != null) {
                return false;
            }
        } else if (!host.equals(other.host)) {
            return false;
        }
        if (isolateJBIClassLoaders != other.isolateJBIClassLoaders) {
            return false;
        }
        if (jmxRMIConnectorPort != other.jmxRMIConnectorPort) {
            return false;
        }
        if (name == null) {
            if (other.name != null) {
                return false;
            }
        } else if (!name.equals(other.name)) {
            return false;
        }
        if (password == null) {
            if (other.password != null) {
                return false;
            }
        } else if (!password.equals(other.password)) {
            return false;
        }
        if (persistenceDuration != other.persistenceDuration) {
            return false;
        }
        if (persistenceFetchSize != other.persistenceFetchSize) {
            return false;
        }
        if (recoveryCorePoolSize != other.recoveryCorePoolSize) {
            return false;
        }
        if (recoveryKeepAliveTime != other.recoveryKeepAliveTime) {
            return false;
        }
        if (recoveryMaxPoolSize != other.recoveryMaxPoolSize) {
            return false;
        }
        if (registryDatabaseDialect == null) {
            if (other.registryDatabaseDialect != null) {
                return false;
            }
        } else if (!registryDatabaseDialect.equals(other.registryDatabaseDialect)) {
            return false;
        }
        if (registryDatabaseDriver == null) {
            if (other.registryDatabaseDriver != null) {
                return false;
            }
        } else if (!registryDatabaseDriver.equals(other.registryDatabaseDriver)) {
            return false;
        }
        if (registryDatabasePassword == null) {
            if (other.registryDatabasePassword != null) {
                return false;
            }
        } else if (!registryDatabasePassword.equals(other.registryDatabasePassword)) {
            return false;
        }
        if (registryDatabaseUrl == null) {
            if (other.registryDatabaseUrl != null) {
                return false;
            }
        } else if (!registryDatabaseUrl.equals(other.registryDatabaseUrl)) {
            return false;
        }
        if (registryDatabaseUser == null) {
            if (other.registryDatabaseUser != null) {
                return false;
            }
        } else if (!registryDatabaseUser.equals(other.registryDatabaseUser)) {
            return false;
        }
        if (registryMode != other.registryMode) {
            return false;
        }
        if (registryPort != other.registryPort) {
            return false;
        }
        if (registryRepository == null) {
            if (other.registryRepository != null) {
                return false;
            }
        } else if (!registryRepository.equals(other.registryRepository)) {
            return false;
        }
        if (registrySynchroPeriod != other.registrySynchroPeriod) {
            return false;
        }
        if (registryTransportTimeout != other.registryTransportTimeout) {
            return false;
        }
        if (repositoryDirectoryPath == null) {
            if (other.repositoryDirectoryPath != null) {
                return false;
            }
        } else if (!repositoryDirectoryPath.equals(other.repositoryDirectoryPath)) {
            return false;
        }
        if (routerQOS == null) {
            if (other.routerQOS != null) {
                return false;
            }
        } else if (!routerQOS.equals(other.routerQOS)) {
            return false;
        }
        if (routerSendAttempt != other.routerSendAttempt) {
            return false;
        }
        if (routerSendDelay != other.routerSendDelay) {
            return false;
        }
        if (routerStrategy == null) {
            if (other.routerStrategy != null) {
                return false;
            }
        } else if (!routerStrategy.equals(other.routerStrategy)) {
            return false;
        }
        if (serverPropertiesUrl == null) {
            if (other.serverPropertiesUrl != null) {
                return false;
            }
        } else if (!serverPropertiesUrl.equals(other.serverPropertiesUrl)) {
            return false;
        }
        if (sslKeyPassword == null) {
            if (other.sslKeyPassword != null) {
                return false;
            }
        } else if (!sslKeyPassword.equals(other.sslKeyPassword)) {
            return false;
        }
        if (sslKeystore == null) {
            if (other.sslKeystore != null) {
                return false;
            }
        } else if (!sslKeystore.equals(other.sslKeystore)) {
            return false;
        }
        if (sslKeystorePassword == null) {
            if (other.sslKeystorePassword != null) {
                return false;
            }
        } else if (!sslKeystorePassword.equals(other.sslKeystorePassword)) {
            return false;
        }
        if (sslTruststore == null) {
            if (other.sslTruststore != null) {
                return false;
            }
        } else if (!sslTruststore.equals(other.sslTruststore)) {
            return false;
        }
        if (sslTruststorePassword == null) {
            if (other.sslTruststorePassword != null) {
                return false;
            }
        } else if (!sslTruststorePassword.equals(other.sslTruststorePassword)) {
            return false;
        }
        if (state != other.state) {
            return false;
        }
        if (subdomainName == null) {
            if (other.subdomainName != null) {
                return false;
            }
        } else if (!subdomainName.equals(other.subdomainName)) {
            return false;
        }
        if (taskTimeout != other.taskTimeout) {
            return false;
        }
        if (tcpConnectionTimeout != other.tcpConnectionTimeout) {
            return false;
        }
        if (tcpPort != other.tcpPort) {
            return false;
        }
        if (tcpReceivers != other.tcpReceivers) {
            return false;
        }
        if (tcpSendTimeout != other.tcpSendTimeout) {
            return false;
        }
        if (tcpSenderEvictableDelay != other.tcpSenderEvictableDelay) {
            return false;
        }
        if (tcpSenderEvictorDelay != other.tcpSenderEvictorDelay) {
            return false;
        }
        if (tcpSenders != other.tcpSenders) {
            return false;
        }
        if (topologyPassPhrase == null) {
            if (other.topologyPassPhrase != null) {
                return false;
            }
        } else if (!topologyPassPhrase.equals(other.topologyPassPhrase)) {
            return false;
        }
        if (transporterQueueMaxSize != other.transporterQueueMaxSize) {
            return false;
        }
        if (transporterQueueOfferingTimeout != other.transporterQueueOfferingTimeout) {
            return false;
        }
        if (user == null) {
            if (other.user != null) {
                return false;
            }
        } else if (!user.equals(other.user)) {
            return false;
        }
        if (userConfiguration == null) {
            if (other.userConfiguration != null) {
                return false;
            }
        } else if (!userConfiguration.equals(other.userConfiguration)) {
            return false;
        }
        if (webservicePort != other.webservicePort) {
            return false;
        }
        if (webservicePrefix == null) {
            if (other.webservicePrefix != null) {
                return false;
            }
        } else if (!webservicePrefix.equals(other.webservicePrefix)) {
            return false;
        }
        if (workDirectoryPath == null) {
            if (other.workDirectoryPath != null) {
                return false;
            }
        } else if (!workDirectoryPath.equals(other.workDirectoryPath)) {
            return false;
        }
        return true;
    }
}
